package com.urbancode.air.plugin.udeploy.resources;
import com.urbancode.air.AirPluginTool
import com.urbancode.ud.client.UDRestClient
import java.util.UUID
import java.util.List
import java.util.Map
import java.util.HashMap
import org.codehaus.jettison.json.JSONObject
public class ResourceHelper {
    def apTool
    def props = []
    def udUser
    def udPass
    def weburl
    def client
    
    public ResourceHelper(def apToolIn) {
        apTool = apToolIn
        props = apTool.getStepProperties()
        udUser = apTool.getAuthTokenUsername()
        udPass = apTool.getAuthToken()
        weburl = System.getenv("AH_WEB_URL")
        client = new UDRestClient(new URI(weburl), udUser, udPass)
        
        com.urbancode.air.XTrustProvider.install()
    }
    
    def checkIfResourceExists() {
        def resourceName = props['resource']
        
        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        
        try {
            JSONObject resourceJSON = client.getResourceByName(resourceName);
            if (resourceJSON == null) {
                throw new IOException("no resource found")
            }
            println "Resource with name ${resourceName} was found."
            apTool.setOutputProperty("exists", "true");
        }
        catch(IOException e) {
            if(e.getMessage().contains("404") || e.getMessage().contains("no resource found")) {
                println "Request was successful but no resource with name ${resourceName} was found."
                apTool.setOutputProperty("exists", "false")
            }
            else {
                println "An error occurred during your request."
                throw new IOException(e);
            }
        }
        apTool.setOutputProperties();
    }
    
    def createResource() {
        def resourceName = props['Name']
        def description = props['Description']
        def childOf = props['ChildOf']
        def parentName = props['Parent']
        
        if (!description) {
            description = ""
        }
        
        def newResource = client.createResource(resourceName, description, childOf, parentName)
        apTool.setOutputProperty("new.resource.name", resourceName)
        apTool.setOutputProperties()
    }
    
    def addToGroup() {
        def resourceName = props['resourceName']
        def resourceGroupPath = props['resourceGroupPath']
        
        if (!resourceName) {
            throw new IllegalArgumentException("no resource name was specified")
        }
        if (!resourceGroupPath) {
            throw new IllegalArgumentException("no resource group path was specified")
        }
        
        println "adding resource $resourceName to group at path $resourceGroupPath"
        client.addResourceToGroup(resourceName, resourceGroupPath)
        
        apTool.setOutputProperty("resource.name", resourceName)
        apTool.setOutputProperty("resource.group.path", resourceGroupPath)
        apTool.setOutputProperties()
    }
    
    def deleteGroup() {
        def resourceGroupPath = props['resourceGroupPath'];
        
        if (!resourceGroupPath) {
            throw new IllegalArgumentException("no resource group path was specified")
        }

        //sanitize trailing slashes of resource group path
        while (resourceGroupPath.endsWith("/")) {
            resourceGroupPath = resourceGroupPath.substring(0, resourceGroupPath.length()-1);
        }

        //create resource group (returns status OK if already exists or Created if successfully created
        client.deleteResourceGroup(resourceGroupPath);
    }

    def createDynamicGroup() {
        def resourceGroupPath = props['resourceGroupPath'];
        def resourceGroupName = props['resourceGroupName'];
        
        if (!resourceGroupPath) {
            throw new IllegalArgumentException("no resource group path was specified")
        }
        if (!resourceGroupName) {
            throw new IllegalArgumentException("no resource group name was specified")
        }

        //sanitize trailing slashes of resource group path
        while (resourceGroupPath.endsWith("/")) {
            resourceGroupPath = resourceGroupPath.substring(0, resourceGroupPath.length()-1);
        }

        //create resource group (returns status OK if already exists or Created if successfully created
        client.createDynamicResourceGroup(resourceGroupPath, resourceGroupName);
        
        apTool.setOutputProperty("resource.group.path", resourceGroupPath)
        apTool.setOutputProperties()
    }

    def addDynamicResourceGroupNameCondition() {
        def resourceGroupPath = props['resourceGroupPath'];

        if (!resourceGroupPath) {
            throw new IllegalArgumentException("no resource group path was specified")
        }

        while(resourceGroupPath.endsWith("/")) {
            resourceGroupPath = resourceGroupPath.substring(0, resourceGroupPath.length()-1);
        }

        while(resourceGroupPath.startsWith("/")) {
            resourceGroupPath = resourceGroupPath.substring(1, resourceGroupPath.length());
        }
        resourceGroupPath="/"+resourceGroupPath;

        def comparisonType = props['comparisonType'];
        def value = props['value'];

        println "Adding rule to " + resourceGroupPath + " : " + comparisonType + " : " + value;

        client.addNameConditionToDynamicGroup(resourceGroupPath, comparisonType, value);
    }

    def addDynamicResourceGroupPropertyCondition() {
        def resourceGroupPath = props['resourceGroupPath'];

        if (!resourceGroupPath) {
            throw new IllegalArgumentException("no resource group path was specified")
        }
        while(resourceGroupPath.endsWith("/")) {
            resourceGroupPath = resourceGroupPath.substring(0, resourceGroupPath.length()-1);
        }

        while(resourceGroupPath.startsWith("/")) {
            resourceGroupPath = resourceGroupPath.substring(1, resourceGroupPath.length());
        }
        resourceGroupPath="/"+resourceGroupPath;

        def propertyName = props['propertyName'];
        def comparisonType = props['comparisonType'];
        def value = props['value'];

        println "Adding rule to " + resourceGroupPath + " : " + comparisonType + " : " + propertyName + " : " +value;

        client.addPropertyConditionToDynamicGroup(resourceGroupPath, comparisonType, value, propertyName);
    }

    def createGroup() {
        def resourceGroupPath = props['resourceGroupPath']
        
        if (!resourceGroupPath) {
            throw new IllegalArgumentException("no resource group path was specified")
        }
        //sanitize trailing slashes of resource group path
        while (resourceGroupPath.endsWith("/")) {
            resourceGroupPath = resourceGroupPath.substring(0, resourceGroupPath.length()-1);
        }
        //add resources to resource group
        def lastSlash = resourceGroupPath.lastIndexOf('/')
        def resourceGroupParent
        def resourceGroupChild
        
        //parse resource group parent and child
        if (lastSlash > 0) {
            resourceGroupParent = resourceGroupPath.substring(0, lastSlash)
            resourceGroupChild = resourceGroupPath.substring(lastSlash + 1)
        }
        else if (lastSlash == 0) {
            resourceGroupParent = "/"
            resourceGroupChild = resourceGroupPath.substring(1)
        }
        else {
            resourceGroupParent = "/"
            resourceGroupChild = "" + resourceGroupPath
        }
        if (resourceGroupPath.charAt(0) != '/') {
            resourceGroupPath = "/" + resourceGroupPath
        }
        //create resource group (returns status OK if already exists or Created if successfully created
        if (resourceGroupParent && resourceGroupChild) {
            client.createResourceGroup(resourceGroupParent, resourceGroupChild)
        }
        
        apTool.setOutputProperty("resource.group.path", resourceGroupPath)
        apTool.setOutputProperties()
    }
    
    def setResourceProperty() {
        def resourceName = props['resource']
        def propName = props['name']
        def propValue = props['value']
        def isSecure = Boolean.valueOf(props['isSecure'])
        
        if (!propName) {
            throw new IllegalArgumentException("no property name was specified")
        }
        if (!propValue) {
            propValue = ""
        }
        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        client.setResourceProperty(resourceName, propName, propValue, isSecure)
    }
    
    def setResourceRoleProperty() {
        def resourceName = props['resource']
        def roleName = props['role']
        def propName = props['name']
        def propValue = props['value']
        
        if (!propName) {
            throw new IllegalArgumentException("no property name was specified")
        }
        if (!propValue) {
            propValue = ""
        }
        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        if (!roleName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        client.setResourceRoleProperty(roleName, resourceName, propName, propValue)
    }
    
    def checkIfResourceHasRole() {
        def resourceName = props['resource']
        def roleName = props['role']
        
        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        if (!roleName) {
            throw new IllegalArgumentException("no role was specified")
        }
        boolean foundRole = false;
        List<String> roles = client.getResourceRolesAsStrings(resourceName)
        for (String role : roles) {
            if (roleName.equals(role)) {
                foundRole = true;
                println "success: found role " + roleName + " on resource " + resourceName
                break;
            }
        }
        if (!foundRole) {
            throw new RuntimeException("The request went through, but the role was not on the resource.")
        }
        
    }
    
    def addRoleToResource() {
        def resourceName = props['resource']
        def roleName = props['role']
        
        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        if (!roleName) {
            throw new IllegalArgumentException("no role was specified")
        }
        Map<String, String> emptyMap = new HashMap<String, String>()
        client.addRoleToResource(resourceName, roleName, emptyMap)
    }
    
    def removeRoleFromResource() {
        def resourceName = props['resource']
        def roleName = props['role']
        
        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        if (!roleName) {
            throw new IllegalArgumentException("no role was specified")
        }
        client.removeRoleFromResource(resourceName, roleName)
    }
    
    def deleteResource() {
        def resourceName = props['resource']
        
        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        client.deleteResource(resourceName)
        println "deleted resource " + resourceName
    }
    
    def deleteAgent() {
        def agentName = props['agent']
        
        if (!agentName) {
            throw new IllegalArgumentException("no agent was specified")
        }
        client.deleteAgent(agentName)
        println "deleted agent " + agentName
    }
    
    def deleteManyResources() {
        String resourceNames = props['resources']
        
        if (!resourceNames) {
            throw new IllegalArgumentException("no resources were specified")
        }
        
        String[] resourceArray = resourceNames.split(",");
        
        for (String resource : resourceArray) {
            client.deleteResource(resource)
            println "deleted resource " + resource
        }
    }
    
    def deleteManyAgents() {
        String agentNames = props['agents']
        
        if (!agentNames) {
            throw new IllegalArgumentException("no agents were specified")
        }
        
        String[] agentArray = agentNames.split(",");
        
        for (String agent : agentArray) {
            client.deleteAgent(agent)
            println "deleted agent " + agent
        }
    }
    
    def getResourceProperty() {
        String resourceName = props['resource']
        String propName = props['name']
        
        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        if (!propName) {
            throw new IllegalArgumentException("no property name was specified")
        }
        
        String propValue = client.getResourceProperty(resourceName, propName)
        
        println "property " + propName + " resolved to " + propValue
        apTool.setOutputProperty(propName, propValue)
        apTool.setOutputProperties()
    }
    
    def getResourceRoleProperty() {
        def resourceName = props['resource']
        def roleName = props['role']
        def propName = props['name']
        
        if (!propName) {
            throw new IllegalArgumentException("no property name was specified")
        }
        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        if (!roleName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        String propValue = client.getResourceRolePropertyForResource(roleName, resourceName, propName)
        println "property " + propName + " resolved to " + propValue
        apTool.setOutputProperty(propName, propValue)
        apTool.setOutputProperties()
    }
    
    def deleteResourceInventoryForComponent() {
        def resourceName = props['resource']
        def versionName = props['version']
        def statusName = props['status']
        def componentNames =
                props['component'].split("\n").findAll{it.trim().length() > 0}.collect{it.trim()}


        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        if (!componentNames) {
            throw new IllegalArgumentException("no components were specified")
        }
        if (!versionName || versionName == "*") {
            versionName = ""
        }
        if (!statusName || statusName == "*") {
            statusName = ""
        }

        
        println "Will remove inventory for the following components: "+componentNames.join(", ")

        componentNames.each{ componentName ->
            try {
                client.deleteResourceInventoryForComponent(
                        resourceName,
                        componentName,
                        versionName,
                        statusName)
            }
            catch (Exception e) {
                throw new RuntimeException("Failed to delete inventory for component, '"+componentName+"'", e)
            }
        }
        println "Deleted all inventory entries that matched the input criteria."
    }
    
    public def addUserToSecurityRole() {
        def userName = props['user']
        def roleName = props['role']
        def resourceName = props['resource']
        
        if (!userName) {
            throw new IllegalArgumentException("no user was specified")
        }
        if (!roleName) {
            throw new IllegalArgumentException("no role was specified")
        }
        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        client.addUserToSecurityRoleForResource(userName, roleName, resourceName)
        println "Added user ${userName} to role ${roleName} for resource ${resourceName}"
    }
    
    public def addGroupToSecurityRole() {
        def groupName = props['group']
        def roleName = props['role']
        def resourceName = props['resource']
        
        if (!groupName) {
            throw new IllegalArgumentException("no group was specified")
        }
        if (!roleName) {
            throw new IllegalArgumentException("no role was specified")
        }
        if (!resourceName) {
            throw new IllegalArgumentException("no resource was specified")
        }
        client.addGroupToSecurityRoleForResource(groupName, roleName, resourceName)
        println "Added group ${groupName} to role ${roleName} for resource ${resourceName}"
    }
}

